@@ -44,9 +44,11 @@ static char *last_msg = NULL;
 static int last_mark_start = 0;
 static int last_mark_end = 0;
 
+static void marked_message_internal (char *msg, int mark_start, int mark_end);
+
 /* Reset the alarm to auto-hide the bar in BAR_TIMEOUT seconds. */
 static void
-reset_alarm ()
+reset_alarm (void)
 {
   alarm (defaults.bar_timeout);
   alarm_signalled = 0;
@@ -60,6 +62,14 @@ hide_bar (rp_screen *s)
     {
       s->bar_is_raised = 0;
       XUnmapWindow (dpy, s->bar_window);
+
+      /* Possibly restore colormap. */
+      if (current_window())
+	{
+	  XUninstallColormap (dpy, s->def_cmap);
+	  XInstallColormap (dpy, current_window()->colormap);
+	}
+
       return 1;
     }
 
@@ -76,6 +86,11 @@ show_bar (rp_screen *s, char *fmt)
       XMapRaised (dpy, s->bar_window);
       update_window_names (s, fmt);
 
+      /* Switch to the default colormap */
+      if (current_window())
+	XUninstallColormap (dpy, current_window()->colormap);
+      XInstallColormap (dpy, s->def_cmap);
+
       reset_alarm();
       return 1;
     }
@@ -96,17 +111,17 @@ bar_x (rp_screen *s, int width)
     case NorthWestGravity:
     case WestGravity:
     case SouthWestGravity:
-      x = s->left;
+      x = s->left + defaults.padding_left;
       break;
     case NorthGravity:
     case CenterGravity:
     case SouthGravity:
-      x = s->left + (s->width - width - defaults.bar_border_width * 2) / 2;
+      x = s->left + (s->width - width - defaults.bar_border_width * 2) / 2 - defaults.padding_right;
       break;
     case NorthEastGravity:
     case EastGravity:
     case SouthEastGravity:
-      x = s->left + s->width - width - defaults.bar_border_width * 2;
+      x = s->left + s->width - width - defaults.bar_border_width * 2 - defaults.padding_right;
       break;
     }
 
@@ -123,19 +138,21 @@ bar_y (rp_screen *s, int height)
     case NorthEastGravity:
     case NorthGravity:
     case NorthWestGravity:
-      y = s->top;
+      y = s->top + defaults.padding_top;
       break;
     case EastGravity:
     case CenterGravity:
     case WestGravity:
       y = s->top + (s->height - height
-           - defaults.bar_border_width * 2) / 2;
+           - defaults.bar_border_width * 2) / 2
+	   - defaults.padding_top;
       break;
     case SouthEastGravity:
     case SouthGravity:
     case SouthWestGravity:
       y = s->top + (s->height - height
-           - defaults.bar_border_width * 2);
+           - defaults.bar_border_width * 2)
+ 	   - defaults.padding_top;
       break;
     }
 
@@ -145,12 +162,19 @@ bar_y (rp_screen *s, int height)
 void
 update_bar (rp_screen *s)
 {
+  if (s->bar_is_raised == BAR_IS_WINDOW_LIST) {
+    update_window_names (s, defaults.window_fmt);
+    return;
+  }
+
   if (s->bar_is_raised == BAR_IS_HIDDEN)
     return;
 
-  show_last_message();
+  redraw_last_message();
 }
 
+/* Note that we use marked_message_internal to avoid resetting the
+   alarm. */
 void
 update_window_names (rp_screen *s, char *fmt)
 {
@@ -165,12 +189,12 @@ update_window_names (rp_screen *s, char *fmt)
   if(defaults.window_list_style == STYLE_ROW)
     {
       get_window_list (fmt, NULL, bar_buffer, &mark_start, &mark_end);
-      marked_message (sbuf_get (bar_buffer), mark_start, mark_end);
+      marked_message_internal (sbuf_get (bar_buffer), mark_start, mark_end);
     }
   else
     {
       get_window_list (fmt, "\n", bar_buffer, &mark_start, &mark_end);
-      marked_message (sbuf_get (bar_buffer), mark_start, mark_end);
+      marked_message_internal (sbuf_get (bar_buffer), mark_start, mark_end);
     }
 
 
@@ -231,7 +255,7 @@ max_line_length (char* msg)
           int current_width;
 
           /* Check if this line is the longest so far. */
-          current_width = XTextWidth (defaults.font, msg + start, i - start);
+          current_width = XmbTextEscapement (defaults.font, msg + start, i - start);
           if(current_width > ret)
             {
               ret = current_width;
@@ -306,22 +330,22 @@ draw_string (rp_screen *s, char *msg)
          line, and move down one line. */
       if (msg[i] == '\n')
         {
-          XDrawString (dpy, s->bar_window, s->normal_gc,
-                       defaults.bar_x_padding,
-                       defaults.bar_y_padding + defaults.font->max_bounds.ascent
-                       +  line_no * line_height,
-                       msg + start, i - start);
+          XmbDrawString (dpy, s->bar_window, defaults.font, s->normal_gc,
+			 defaults.bar_x_padding,
+			 defaults.bar_y_padding + rp_font_ascent
+			 +  line_no * line_height,
+			 msg + start, i - start);
           line_no++;
           start = i + 1;
         }
     }
 
   /* Print the last line. */
-  XDrawString (dpy, s->bar_window, s->normal_gc,
-               defaults.bar_x_padding,
-               defaults.bar_y_padding + defaults.font->max_bounds.ascent
-               +  line_no * line_height,
-               msg + start, strlen (msg) - start);
+  XmbDrawString (dpy, s->bar_window, defaults.font, s->normal_gc,
+		 defaults.bar_x_padding,
+		 defaults.bar_y_padding + rp_font_ascent
+		 +  line_no * line_height,
+		 msg + start, strlen (msg) - start);
 
   XSync (dpy, False);
 }
@@ -359,6 +383,8 @@ correct_mark (int msg_len, int *mark_start, int *mark_end)
 static void
 prepare_bar (rp_screen *s, int width, int height)
 {
+  width = width < s->width ? width : s->width;
+  height = height < s->height ? height : s->height;
   XMoveResizeWindow (dpy, s->bar_window,
                      bar_x (s, width), bar_y (s, height),
                      width, height);
@@ -368,6 +394,11 @@ prepare_bar (rp_screen *s, int width, int height)
     {
       s->bar_is_raised = BAR_IS_MESSAGE;
       XMapRaised (dpy, s->bar_window);
+
+      /* Switch to the default colormap */
+      if (current_window())
+	XUninstallColormap (dpy, current_window()->colormap);
+      XInstallColormap (dpy, s->def_cmap);
     }
 
   XRaiseWindow (dpy, s->bar_window);
@@ -412,11 +443,11 @@ get_mark_box (char *msg, size_t mark_start, size_t mark_end,
   if (mark_start == 0 || start_pos_in_line == 0)
     start = 0;
   else
-    start = XTextWidth (defaults.font,
+    start = XmbTextEscapement (defaults.font,
                         &msg[start_line_beginning],
                         start_pos_in_line) + defaults.bar_x_padding;
 
-  end = XTextWidth (defaults.font,
+  end = XmbTextEscapement (defaults.font,
                     &msg[end_line_beginning],
                     end_pos_in_line) + defaults.bar_x_padding * 2;
 
@@ -489,18 +520,23 @@ update_last_message (char *msg, int mark_start, int mark_end)
 
 void
 marked_message (char *msg, int mark_start, int mark_end)
+{
+  /* Schedule the bar to be hidden after some amount of time. */
+  reset_alarm ();
+  marked_message_internal (msg, mark_start, mark_end);
+}
+
+static void
+marked_message_internal (char *msg, int mark_start, int mark_end)
 {
   rp_screen *s = current_screen ();
   int num_lines;
   int width;
   int height;
 
-  PRINT_DEBUG (("msg = %s\n", msg));
+  PRINT_DEBUG (("msg = %s\n", msg?msg:"NULL"));
   PRINT_DEBUG (("mark_start = %d, mark_end = %d\n", mark_start, mark_end));
 
-  /* Schedule the bar to be hidden after some amount of time. */
-  reset_alarm ();
-
   /* Calculate the width and height of the window. */
   num_lines = count_lines (msg, strlen(msg));
   width = defaults.bar_x_padding * 2 + max_line_length(msg);
@@ -518,8 +554,10 @@ marked_message (char *msg, int mark_start, int mark_end)
   update_last_message (msg, mark_start, mark_end);
 }
 
+/* Use this just to update the bar. show_last_message will draw it and
+   leave it up for a period of time. */
 void
-show_last_message ()
+redraw_last_message (void)
 {
   char *msg;
 
@@ -530,13 +568,20 @@ show_last_message ()
      marked_message's msg arg would have been the same as
      last_msg.  */
   msg = xstrdup (last_msg);
-  marked_message (msg, last_mark_start, last_mark_end);
+  marked_message_internal (msg, last_mark_start, last_mark_end);
   free (msg);
 }
 
+void
+show_last_message (void)
+{
+  redraw_last_message();
+  reset_alarm();
+}
+
 /* Free any memory associated with the bar. */
 void
-free_bar ()
+free_bar (void)
 {
   if (last_msg)
     free (last_msg);
