@@ -220,7 +220,7 @@ struct rp_defaults
   int padding_top;
   int padding_bottom;
 
-  XFontStruct *font;
+  XFontSet font;
   char *font_string;
 
   char *fgcolor_string;
@@ -249,6 +249,12 @@ struct rp_defaults
 
   /* How many frame sets to remember when undoing. */
   int maxundos;
+
+  /* The name of the top level keymap */
+  char *top_kmap;
+
+  /* Frame indicator format */
+  char *frame_fmt;
 };
 
 /* Information about a child process. */
