@@ -42,27 +42,28 @@
    action.c which need to forward events to other windows. */
 XEvent rp_current_event;
 
+/* RAISED is non zero if a raised message should be used 0 for a map message. */
 void
-show_rudeness_raise_msg (rp_window *win)
+show_rudeness_msg (rp_window *win, int raised)
 {
   rp_group *g = groups_find_group_by_window (win);
   rp_window_elem *elem = group_find_window (&g->mapped_windows, win);
   if (g == rp_current_group)
     {
       if (win->transient)
-        marked_message_printf (0, 0, MESSAGE_RAISE_TRANSIENT,
+        marked_message_printf (0, 0, raised ? MESSAGE_RAISE_TRANSIENT:MESSAGE_MAP_TRANSIENT,
                                elem->number, window_name (win));
       else
-        marked_message_printf (0, 0, MESSAGE_RAISE_WINDOW,
+        marked_message_printf (0, 0, raised ? MESSAGE_RAISE_WINDOW:MESSAGE_MAP_WINDOW,
                                elem->number, window_name (win));
     }
   else
     {
       if (win->transient)
-        marked_message_printf (0, 0, MESSAGE_RAISE_TRANSIENT_GROUP,
+        marked_message_printf (0, 0, raised ? MESSAGE_RAISE_TRANSIENT_GROUP:MESSAGE_MAP_TRANSIENT_GROUP,
                                elem->number, window_name (win), g->name);
       else
-        marked_message_printf (0, 0, MESSAGE_RAISE_WINDOW_GROUP,
+        marked_message_printf (0, 0, raised ? MESSAGE_RAISE_WINDOW_GROUP:MESSAGE_MAP_WINDOW_GROUP,
                                elem->number, window_name (win), g->name);
     }
 }
@@ -196,7 +197,7 @@ map_request (XEvent *ev)
               || (rp_honour_normal_raise && !win->transient))
             set_active_window (win);
           else
-            show_rudeness_raise_msg (win);
+            show_rudeness_msg (win, 1);
         }
       break;
     }
@@ -268,7 +269,7 @@ configure_request (XConfigureRequestEvent *e)
                 }
               else if (current_window() != win)
                 {
-                  show_rudeness_raise_msg (win);
+                  show_rudeness_msg (win, 1);
                 }
 
             }
@@ -382,11 +383,11 @@ static void
 handle_key (KeySym ks, unsigned int mod, rp_screen *s)
 {
   rp_action *key_action;
-  rp_keymap *map = find_keymap (TOP_KEYMAP);
+  rp_keymap *map = find_keymap (defaults.top_kmap);
 
   if (map == NULL)
     {
-      PRINT_ERROR (("Unable to find " TOP_KEYMAP " keymap\n"));
+      PRINT_ERROR (("Unable to find %s keymap\n", defaults.top_kmap));
       return;
     }
 
@@ -424,7 +425,7 @@ handle_key (KeySym ks, unsigned int mod, rp_screen *s)
     }
   else
     {
-      PRINT_ERROR(("Impossible: No matching key\n"));
+      PRINT_DEBUG(("Impossible: No matching key"));
     }
   XAllowEvents (dpy, AsyncKeyboard, CurrentTime);
 }
@@ -475,8 +476,7 @@ execute_remote_command (Window w)
 
   if (status != Success || req == NULL)
     {
-      PRINT_DEBUG (("Couldn't get RP_COMMAND Property\n"));
-      return NULL;
+      return cmdret_new (RET_FAILURE, "Couldn't get RP_COMMAND Property");
     }
 
   /* XGetWindowProperty always allocates one extra byte even if
@@ -490,8 +490,7 @@ execute_remote_command (Window w)
 
   if (status != Success || req == NULL)
     {
-      PRINT_DEBUG (("Couldn't get RP_COMMAND Property\n"));
-      return NULL;
+      return cmdret_new (RET_FAILURE, "Couldn't get RP_COMMAND Property");
     }
 
   PRINT_DEBUG (("command: %s\n", req));
@@ -604,12 +603,14 @@ property_notify (XEvent *ev)
 
           PRINT_DEBUG (("updating _NET_WM_PID\n"));
           child_info = get_child_info(win->w);
-          if (child_info)
+          if (child_info && !child_info->window_mapped)
             {
               if (child_info->frame)
                 {
                   PRINT_DEBUG (("frame=%p\n", child_info->frame));
                   win->intended_frame_number = child_info->frame->number;
+		  /* Only map the first window in the launch frame. */
+		  child_info->window_mapped = 1;
                 }
               /* TODO: also adopt group information? */
             }
@@ -618,8 +619,8 @@ property_notify (XEvent *ev)
           {
           case XA_WM_NAME:
             PRINT_DEBUG (("updating window name\n"));
-            update_window_name (win);
-            update_window_names (win->scr, defaults.window_fmt);
+            if (update_window_name (win))
+	      update_window_names (win->scr, defaults.window_fmt);
             break;
 
           case XA_WM_NORMAL_HINTS:
@@ -659,7 +660,8 @@ colormap_notify (XEvent *ev)
       XGetWindowAttributes (dpy, win->w, &attr);
       win->colormap = attr.colormap;
 
-      if (win == current_window())
+      if (win == current_window()
+	  && !current_screen()->bar_is_raised)
         {
           XInstallColormap (dpy, win->colormap);
         }
@@ -777,7 +779,7 @@ selection_request (XSelectionRequestEvent *rq)
 }
 
 static void
-selection_clear ()
+selection_clear (void)
 {
   if (selection.text)
     free (selection.text);
@@ -881,7 +883,7 @@ delegate_event (XEvent *ev)
 }
 
 static void
-handle_signals ()
+handle_signals (void)
 {
   /* An alarm means we need to hide the popup windows. */
   if (alarm_signalled > 0)
@@ -929,7 +931,7 @@ handle_signals ()
       putenv(current_screen()->display_string);
       unhide_all_windows();
       XSync(dpy, False);
-      execlp(rp_exec_newwm, rp_exec_newwm, 0);
+      execlp(rp_exec_newwm, rp_exec_newwm, NULL);
 
       /* Failed. Clean up. */
       PRINT_ERROR (("exec %s ", rp_exec_newwm));
@@ -965,7 +967,7 @@ handle_signals ()
 
 /* The main loop. */
 void
-listen_for_events ()
+listen_for_events (void)
 {
   int x_fd;
   fd_set fds;
