@@ -74,14 +74,58 @@ rp_mask_to_x11_mask (unsigned int mask)
 }
 
 
+/* /\* The caller is responsible for freeing the keycodes. *\/ */
+/* KeyCode * */
+/* keysym_to_keycodes (KeySym sym, int *n_returned) */
+/* { */
+/*   int min_code, max_code; */
+/*   int syms_per_code; */
+/*   KeySym *syms; */
+/*   KeyCode *codes; */
+/*   int code, code_col; */
+
+/*   XDisplayKeycodes (dpy, &min_code, &max_code); */
+/*   syms = XGetKeyboardMapping (dpy, */
+/* 			      min_code, max_code - min_code + 1, */
+/* 			      &syms_per_code); */
+
+/*   *n_returned = 0; */
+/*   codes = (KeyCode *)xmalloc (sizeof(KeyCode) * n_returned); */
+/*   for (code = min_code; code < max_code; code++) */
+/*     for (code_col = 0; code_col < syms_per_code; code_col++) */
+/*       {       */
+/* 	int s = syms[((code - min_code) * syms_per_code) + code_col]; */
+
+/* 	if (sym == s) */
+/* 	  { */
+/* 	    n_returned++; */
+/* 	    codes = (KeyCode *)xrealloc (sizeof(KeyCode) * n_returned); */
+/* 	    codes[n_returned-1] = code; */
+/* 	  } */
+/*       }	 */
+      
+/*   XFree ((char *) syms); */
+
+/*   if (n_returned > 0) */
+/*     return codes; */
+/*   else  */
+/*     { */
+/*       xfree (codes) */
+/*       return NULL; */
+/*     } */
+/* } */
+
 /* Figure out what keysyms are attached to what modifiers */
 void
-update_modifier_map ()
+update_modifier_map (void)
 {
   unsigned int modmasks[] =
     { Mod1Mask, Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask };
   int row, col; /* The row and column in the modifier table.  */
   XModifierKeymap *mods;
+  int min_code, max_code;
+  int syms_per_code;
+  KeySym *syms;
 
   rp_modifier_info.meta_mod_mask = 0;
   rp_modifier_info.alt_mod_mask = 0;
@@ -90,6 +134,10 @@ update_modifier_map ()
   rp_modifier_info.num_lock_mask = 0;
   rp_modifier_info.scroll_lock_mask = 0;
 
+  XDisplayKeycodes (dpy, &min_code, &max_code);
+  syms = XGetKeyboardMapping (dpy,
+			      min_code, max_code - min_code + 1,
+			      &syms_per_code);
   mods = XGetModifierMapping (dpy);
 
   for (row=3; row < 8; row++)
@@ -97,52 +145,64 @@ update_modifier_map ()
       {
         KeyCode code = mods->modifiermap[(row * mods->max_keypermod) + col];
 
+	PRINT_DEBUG (("row: %d col: %d code: %d\n", row, col, code));
+
         if (code == 0) continue;
 
-        switch (XKeycodeToKeysym(dpy, code, 0))
-          {
-          case XK_Meta_L:
-          case XK_Meta_R:
-            rp_modifier_info.meta_mod_mask |= modmasks[row - 3];
-            PRINT_DEBUG (("Found Meta on %d\n",
-                         rp_modifier_info.meta_mod_mask));
-            break;
+	/* Are any of this keycode's keysyms a meta key?  */
+	{
+	  int code_col;
 
-          case XK_Alt_L:
-          case XK_Alt_R:
-            rp_modifier_info.alt_mod_mask |= modmasks[row - 3];
-            PRINT_DEBUG (("Found Alt on %d\n",
-                         rp_modifier_info.alt_mod_mask));
-            break;
+	  for (code_col = 0; code_col < syms_per_code; code_col++)
+	    {
+	      int sym = syms[((code - min_code) * syms_per_code) + code_col];
 
-          case XK_Super_L:
-          case XK_Super_R:
-            rp_modifier_info.super_mod_mask |= modmasks[row - 3];
-            PRINT_DEBUG (("Found Super on %d\n",
-                         rp_modifier_info.super_mod_mask));
-            break;
+	      switch (sym)
+		{
+		case XK_Meta_L:
+		case XK_Meta_R:
+		  rp_modifier_info.meta_mod_mask |= modmasks[row - 3];
+		  PRINT_DEBUG (("Found Meta on %d\n",
+				rp_modifier_info.meta_mod_mask));
+		  break;
 
-          case XK_Hyper_L:
-          case XK_Hyper_R:
-            rp_modifier_info.hyper_mod_mask |= modmasks[row - 3];
-            PRINT_DEBUG (("Found Hyper on %d\n",
-                         rp_modifier_info.hyper_mod_mask));
-            break;
+		case XK_Alt_L:
+		case XK_Alt_R:
+		  rp_modifier_info.alt_mod_mask |= modmasks[row - 3];
+		  PRINT_DEBUG (("Found Alt on %d\n",
+				rp_modifier_info.alt_mod_mask));
+		  break;
 
-          case XK_Num_Lock:
-            rp_modifier_info.num_lock_mask |= modmasks[row - 3];
-            PRINT_DEBUG (("Found NumLock on %d\n",
-                         rp_modifier_info.num_lock_mask));
-            break;
+		case XK_Super_L:
+		case XK_Super_R:
+		  rp_modifier_info.super_mod_mask |= modmasks[row - 3];
+		  PRINT_DEBUG (("Found Super on %d\n",
+				rp_modifier_info.super_mod_mask));
+		  break;
 
-          case XK_Scroll_Lock:
-            rp_modifier_info.scroll_lock_mask |= modmasks[row - 3];
-            PRINT_DEBUG (("Found ScrollLock on %d\n",
-                         rp_modifier_info.scroll_lock_mask));
-            break;
-          default:
-            break;
-          }
+		case XK_Hyper_L:
+		case XK_Hyper_R:
+		  rp_modifier_info.hyper_mod_mask |= modmasks[row - 3];
+		  PRINT_DEBUG (("Found Hyper on %d\n",
+				rp_modifier_info.hyper_mod_mask));
+		  break;
+
+		case XK_Num_Lock:
+		  rp_modifier_info.num_lock_mask |= modmasks[row - 3];
+		  PRINT_DEBUG (("Found NumLock on %d\n",
+				rp_modifier_info.num_lock_mask));
+		  break;
+
+		case XK_Scroll_Lock:
+		  rp_modifier_info.scroll_lock_mask |= modmasks[row - 3];
+		  PRINT_DEBUG (("Found ScrollLock on %d\n",
+				rp_modifier_info.scroll_lock_mask));
+		  break;
+		default:
+		  break;
+		}
+	    }
+	}
       }
 
   /* Stolen from Emacs 21.0.90 - xterm.c */
@@ -160,6 +220,7 @@ update_modifier_map ()
       rp_modifier_info.alt_mod_mask &= ~rp_modifier_info.meta_mod_mask;
     }
 
+  XFree ((char *) syms);
   XFreeModifiermap (mods);
 }
 
@@ -319,8 +380,8 @@ read_key (KeySym *keysym, unsigned int *modifiers, char *keysym_name, int len)
 static void
 update_input_window (rp_screen *s, rp_input_line *line)
 {
-  int   prompt_width = XTextWidth (defaults.font, line->prompt, strlen (line->prompt));
-  int   input_width  = XTextWidth (defaults.font, line->buffer, line->length);
+  int   prompt_width = XmbTextEscapement (defaults.font, line->prompt, strlen (line->prompt));
+  int   input_width  = XmbTextEscapement (defaults.font, line->buffer, line->length);
   int   total_width;
   GC lgc;
   XGCValues gv;
@@ -341,17 +402,17 @@ update_input_window (rp_screen *s, rp_input_line *line)
   XClearWindow (dpy, s->input_window);
   XSync (dpy, False);
 
-  XDrawString (dpy, s->input_window, s->normal_gc,
-               defaults.bar_x_padding,
-               defaults.bar_y_padding + defaults.font->max_bounds.ascent,
-               line->prompt,
-               strlen (line->prompt));
+  XmbDrawString (dpy, s->input_window, defaults.font, s->normal_gc,
+		 defaults.bar_x_padding,
+		 defaults.bar_y_padding + rp_font_ascent,
+		 line->prompt,
+		 strlen (line->prompt));
 
-  XDrawString (dpy, s->input_window, s->normal_gc,
-               defaults.bar_x_padding + prompt_width,
-               defaults.bar_y_padding + defaults.font->max_bounds.ascent,
-               line->buffer,
-               line->length);
+  XmbDrawString (dpy, s->input_window, defaults.font, s->normal_gc,
+		 defaults.bar_x_padding + prompt_width,
+		 defaults.bar_y_padding + rp_font_ascent,
+		 line->buffer,
+		 line->length);
 
   gv.function = GXxor;
   gv.foreground = s->fg_color ^ s->bg_color;
@@ -359,9 +420,9 @@ update_input_window (rp_screen *s, rp_input_line *line)
 
   /* Draw a cheap-o cursor - MkII */
   XFillRectangle (dpy, s->input_window, lgc,
-                  defaults.bar_x_padding + prompt_width + XTextWidth (defaults.font, line->buffer, line->position),
+                  defaults.bar_x_padding + prompt_width + XmbTextEscapement (defaults.font, line->buffer, line->position),
                   defaults.bar_y_padding,
-                  XTextWidth (defaults.font, &line->buffer[line->position], 1),
+                  XmbTextEscapement (defaults.font, &line->buffer[line->position], 1),
                   FONT_HEIGHT (defaults.font));
 
   XFlush (dpy);
@@ -369,7 +430,7 @@ update_input_window (rp_screen *s, rp_input_line *line)
 }
 
 void
-ring_bell ()
+ring_bell (void)
 {
 #ifdef VISUAL_BELL
   GC lgc;
@@ -436,6 +497,11 @@ get_more_input (char *prompt, char *preinput,
   /* We don't want to draw overtop of the program bar. */
   hide_bar (s);
 
+  /* Switch to the default colormap. */
+  if (current_window())
+    XUninstallColormap (dpy, current_window()->colormap);
+  XInstallColormap (dpy, s->def_cmap);
+
   XMapWindow (dpy, s->input_window);
   XRaiseWindow (dpy, s->input_window);
   XClearWindow (dpy, s->input_window);
@@ -486,5 +552,12 @@ get_more_input (char *prompt, char *preinput,
   XUngrabKeyboard (dpy, CurrentTime);
   XUnmapWindow (dpy, s->input_window);
 
+  /* Possibly restore colormap. */
+  if (current_window())
+    {
+      XUninstallColormap (dpy, s->def_cmap);
+      XInstallColormap (dpy, current_window()->colormap);
+    }
+
   return final_input;
 }
