@@ -1,5 +1,5 @@
 /* Function prototypes.
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -29,9 +29,9 @@ char *get_more_input (char *prompt, char *preinput, completion_fn fn);
 int read_key (KeySym *keysym, unsigned int *modifiers, char *keysym_name, int len);
 unsigned int x11_mask_to_rp_mask (unsigned int mask);
 unsigned int rp_mask_to_x11_mask (unsigned int mask);
-void update_modifier_map ();
+void update_modifier_map (void);
 void grab_key (KeySym keysym, unsigned int modifiers, Window grab_window);
 
-void ring_bell ();
+void ring_bell (void);
 
 #endif /* ! _RATPOISON_INPUT_H */
