@@ -151,6 +151,35 @@ xsprintf (char *fmt, ...)
   return buffer;
 }
 
+/* strtok but do it for whitespace and be locale compliant. */
+char *
+strtok_ws (char *s)
+{
+  char *nonws;
+  static char *pointer = NULL;
+
+  if (s)
+    pointer = s;
+  
+  /* skip to first non-whitespace char. */
+  while (*pointer && isspace (*pointer)) pointer++;
+
+  /* If we reached the end of the string here then there is no more
+     data. */
+  if (*pointer == 0)
+    return NULL;
+
+  /* Now skip to the end of the data. */
+  nonws = pointer;
+  while (*pointer && !isspace (*pointer)) pointer++;
+  if (*pointer)
+    {
+      *pointer = 0;
+      pointer++;
+    }
+  return nonws;
+}
+
 /* A case insensitive strncmp. */
 int
 str_comp (char *s1, char *s2, int len)
@@ -163,19 +192,19 @@ str_comp (char *s1, char *s2, int len)
   return 1;
 }
 
-void
+static void
 sighandler (int signum)
 {
   kill_signalled++;
 }
 
-void
+static void
 hup_handler (int signum)
 {
   hup_signalled++;
 }
 
-void
+static void
 alrm_handler (int signum)
 {
   alarm_signalled++;
@@ -184,7 +213,7 @@ alrm_handler (int signum)
 /* Check for child processes that have quit but haven't been
    acknowledged yet. Update their structure. */
 void
-check_child_procs ()
+check_child_procs (void)
 {
   rp_child_info *cur;
   int pid, status;
@@ -221,7 +250,7 @@ chld_handler (int signum)
   errno = serrno;
 }
 
-int
+static int
 handler (Display *d, XErrorEvent *e)
 {
   char error_msg[100];
@@ -280,17 +309,17 @@ set_sig_handler (int sig, void (*action)(int))
     }
 }
 
-void
-print_version ()
+static void
+print_version (void)
 {
-  printf ("%s %s\n", PACKAGE, VERSION);
-  printf ("Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts\n\n");
+  printf ("%s %s (built %s %s)\n", PACKAGE, VERSION, __DATE__, __TIME__);
+  printf ("Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Shawn Betts\n\n");
 
   exit (EXIT_SUCCESS);
 }
 
-void
-print_help ()
+static void
+print_help (void)
 {
   printf ("Help for %s %s\n\n", PACKAGE, VERSION);
   printf ("-h, --help            Display this help screen\n");
@@ -425,7 +454,7 @@ read_startup_files (char *alt_rcfile)
 /* Odd that we spend so much code on making sure the silly welcome
    message is correct. Oh well... */
 static void
-show_welcome_message ()
+show_welcome_message (void)
 {
   rp_action *help_action;
   char *prefix, *help;
@@ -463,8 +492,10 @@ show_welcome_message ()
 }
 
 static void
-init_defaults ()
+init_defaults (void)
 {
+  defaults.top_kmap = xstrdup(TOP_KEYMAP);
+
   defaults.win_gravity     = NorthWestGravity;
   defaults.trans_gravity   = CenterGravity;
   defaults.maxsize_gravity = CenterGravity;
@@ -485,13 +516,14 @@ init_defaults ()
   defaults.padding_top    = 0;
   defaults.padding_bottom = 0;
 
-  defaults.font = XLoadQueryFont (dpy, DEFAULT_FONT);
+  defaults.font = load_query_font_set (dpy, DEFAULT_FONT);
   if (defaults.font == NULL)
     {
       fprintf (stderr, "ratpoison: Cannot load font %s.\n", DEFAULT_FONT);
       exit (EXIT_FAILURE);
     }
   defaults.font_string = xstrdup (DEFAULT_FONT);
+  set_extents_of_fontset (defaults.font);
 
   defaults.fgcolor_string = xstrdup ("black");
   defaults.bgcolor_string = xstrdup ("white");
@@ -500,6 +532,7 @@ init_defaults ()
 
   defaults.window_fmt = xstrdup ("%n%s%t");
   defaults.info_fmt = xstrdup ("(%H, %W) %n(%t)");
+  defaults.frame_fmt = xstrdup ("Current Frame");
 
   defaults.win_name = WIN_NAME_TITLE;
   defaults.startup_message = 1;
@@ -525,6 +558,7 @@ main (int argc, char *argv[])
   char *alt_rcfile = NULL;
 
   myargv = argv;
+  setlocale(LC_CTYPE, "");
 
   /* Parse the arguments */
   while (1)
@@ -718,7 +752,7 @@ free_screen (rp_screen *s)
 }
 
 void
-clean_up ()
+clean_up (void)
 {
   int i;
 
@@ -753,9 +787,36 @@ clean_up ()
 
   free_xinerama();
 
-  XFreeFont (dpy, defaults.font);
+  XFreeFontSet (dpy, defaults.font);
   free (defaults.window_fmt);
 
   XSetInputFocus (dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
   XCloseDisplay (dpy);
 }
+
+void
+set_extents_of_fontset (XFontSet font)
+{
+  XFontSetExtents *extent;
+  extent = XExtentsOfFontSet(font);
+  rp_font_ascent = extent->max_logical_extent.height * 9 / 10;
+  rp_font_descent = extent->max_logical_extent.height / 5;
+  rp_font_width = extent->max_logical_extent.width;
+}
+
+XFontSet load_query_font_set (Display *disp, const char *fontset_name)
+{
+  XFontSet fontset;
+  int  missing_charset_count;
+  char **missing_charset_list;
+  char *def_string;
+
+  fontset = XCreateFontSet(disp, fontset_name,
+                           &missing_charset_list, &missing_charset_count,
+                           &def_string);
+  if (missing_charset_count) {
+    PRINT_DEBUG (("Missing charsets in FontSet(%s) creation.\n", fontset_name));
+    XFreeStringList(missing_charset_list);
+  }
+  return fontset;
+}
