@@ -37,7 +37,7 @@ static char **unmanaged_window_list = NULL;
 static int num_unmanaged_windows = 0;
 
 void
-clear_unmanaged_list ()
+clear_unmanaged_list (void)
 {
   if (unmanaged_window_list)
     {
@@ -54,7 +54,7 @@ clear_unmanaged_list ()
 }
 
 char *
-list_unmanaged_windows ()
+list_unmanaged_windows (void)
 {
   char *tmp = NULL;
   if (unmanaged_window_list)
@@ -110,12 +110,12 @@ grab_top_level_keys (Window w)
   XGrabKey(dpy, AnyKey, AnyModifier, w, True,
            GrabModeAsync, GrabModeAsync);
 #else
-  rp_keymap *map = find_keymap (TOP_KEYMAP);
+  rp_keymap *map = find_keymap (defaults.top_kmap);
   int i;
 
   if (map == NULL)
     {
-      PRINT_ERROR (("Unable to find " TOP_KEYMAP " keymap\n"));
+      PRINT_ERROR (("Unable to find %s level keymap\n", defaults.top_kmap));
       return;
     }
 
@@ -135,7 +135,7 @@ ungrab_top_level_keys (Window w)
 }
 
 void
-ungrab_keys_all_wins ()
+ungrab_keys_all_wins (void)
 {
   rp_window *cur;
 
@@ -147,7 +147,7 @@ ungrab_keys_all_wins ()
 }
 
 void
-grab_keys_all_wins ()
+grab_keys_all_wins (void)
 {
   rp_window *cur;
 
@@ -159,7 +159,7 @@ grab_keys_all_wins ()
 }
 
 rp_screen*
-current_screen ()
+current_screen (void)
 {
   int i;
 
@@ -198,33 +198,29 @@ update_normal_hints (rp_window *win)
 static char *
 get_wmname (Window w)
 {
-  Atom actual_type;
-  int actual_format;
-  int status;
-  unsigned long n;
-  unsigned long bytes_after;
   unsigned char *name = NULL;
   char *ret;
+  XTextProperty text_prop;
+  int n;
+  char** cl;
 
-  status = XGetWindowProperty (dpy, w, wm_name, 0L, 100L, False,
-                               XA_STRING, &actual_type, &actual_format,
-                               &n, &bytes_after, &name);
-
-  PRINT_DEBUG (("XGetWindowProperty: %d %ld %d %ld %ld '%s'\n", status, actual_type,
-                actual_format, n, bytes_after, name));
-
-  if (status != Success || name == NULL)
-    {
-      PRINT_DEBUG (("I can't get the WMName.\n"));
-      return NULL;
-    }
-
-  if (n == 0)
-    {
-      PRINT_DEBUG (("I can't get the WMName.\n"));
-      XFree (name);
-      return NULL;
+  if (XGetWMName(dpy, w, &text_prop) != 0) {
+    if (text_prop.encoding == XA_STRING) {
+      name = (char *)text_prop.value;
+    } else {
+      XmbTextPropertyToTextList(dpy, &text_prop, &cl, &n);
+      if (cl) {
+        name = strdup(cl[0]);
+        XFreeStringList(cl);
+      } else {
+        PRINT_DEBUG (("I can't get the WMName.\n"));
+        return NULL;
+      }
     }
+  } else {
+    PRINT_DEBUG (("I can't get the WMName.\n"));
+    return NULL;
+  }
 
   PRINT_DEBUG (("WM_NAME: '%s'\n", name));
 
@@ -304,15 +300,18 @@ get_res_class (Window w)
   return name;
 }
 
-/* Reget the WM_NAME property for the window and update its name. */
+/* Reget the WM_NAME property for the window and update its
+   name. Return 1 if the name changed. */
 int
 update_window_name (rp_window *win)
 {
   char *newstr;
+  int changed = 0;
 
   newstr = get_wmname (win->w);
   if (newstr != NULL)
     {
+      changed = changed || win->wm_name == NULL || strcmp (newstr, win->wm_name);
       free (win->wm_name);
       win->wm_name = newstr;
     }
@@ -320,6 +319,7 @@ update_window_name (rp_window *win)
   newstr = get_res_class (win->w);
   if (newstr != NULL)
     {
+      changed = changed || win->res_class == NULL || strcmp (newstr, win->res_class);
       free (win->res_class);
       win->res_class = newstr;
     }
@@ -327,11 +327,12 @@ update_window_name (rp_window *win)
   newstr = get_res_name (win->w);
   if (newstr != NULL)
     {
+      changed = changed || win->res_name == NULL || strcmp (newstr, win->res_name);
       free (win->res_name);
       win->res_name = newstr;
     }
 
-  return 1;
+  return changed;
 }
 
 /* Send an artificial configure event to the window. */
@@ -839,14 +840,7 @@ map_window (rp_window *win)
       || (rp_honour_normal_map && !win->transient))
     set_active_window (win);
   else
-    {
-      if (win->transient)
-        marked_message_printf (0, 0, MESSAGE_MAP_TRANSIENT,
-                               win->number, window_name (win));
-      else
-        marked_message_printf (0, 0, MESSAGE_MAP_WINDOW,
-                               win->number, window_name (win));
-    }
+    show_rudeness_msg (win, 0);
 }
 
 void
@@ -879,7 +873,7 @@ unhide_window (rp_window *win)
 }
 
 void
-unhide_all_windows ()
+unhide_all_windows (void)
 {
   struct list_head *tmp, *iter;
   rp_window *win;
