@@ -41,7 +41,7 @@ update_last_access (rp_frame *frame)
 }
 
 rp_frame *
-current_frame ()
+current_frame (void)
 {
   rp_screen *s = current_screen();
   return screen_get_frame (s, s->current_frame);
@@ -172,7 +172,7 @@ create_initial_frame (rp_screen *screen)
 }
 
 void
-init_frame_lists ()
+init_frame_lists (void)
 {
   int i;
 
@@ -189,7 +189,7 @@ init_frame_list (rp_screen *screen)
 }
 
 rp_frame *
-find_last_frame ()
+find_last_frame (void)
 {
   rp_frame *cur, *last = NULL;
   int last_access = -1;
@@ -245,7 +245,7 @@ find_frame_prev (rp_frame *frame)
 }
 
 rp_window *
-current_window ()
+current_window (void)
 {
   return find_window_number (current_frame()->win_number);
 }
@@ -382,7 +382,7 @@ h_split_frame (rp_frame *frame, int pixels)
 }
 
 void
-remove_all_splits ()
+remove_all_splits (void)
 {
   struct list_head *tmp, *iter;
   rp_screen *s = current_screen();
@@ -871,6 +871,61 @@ set_active_frame (rp_frame *frame)
     }
 }
 
+void
+exchange_with_frame (rp_screen *s, rp_frame *cur, rp_frame *frame)
+{
+  rp_window *win,*last_win;
+
+  /* As a frame does not tell which screen it belongs to,
+   * we could only make sure the frames are not on different
+   * screens when xinerama is not available, if both have
+   * windows in it. As this is not enough the caller will
+   * have to check this. As I only plan to implement 
+   * exchange_{left,right,up,down}, this will be a nop...*/
+  if (frame == NULL || frame == cur) return;
+
+  /* Exchange the windows in the frames */
+  win = find_window_number (cur->win_number);
+  last_win = set_frames_window (frame, win);
+  set_frames_window (cur, last_win);
+
+  /* Make sure the windows comes up full screen */
+  if (last_win)
+    maximize (last_win);
+  if (win)
+    {
+    maximize (win);
+    /* Make sure the program bar is always on the top */
+    update_window_names (win->scr, defaults.window_fmt);
+  }
+
+  /* Make the switch */
+  update_last_access (frame);
+
+  if (s->current_frame == cur->number)
+    {
+      s->current_frame = frame->number;
+      /* mark it as active */
+      show_frame_indicator();
+    }
+
+  update_bar (s);
+
+
+  XSync (dpy, False);
+
+  hook_run (&rp_switch_frame_hook);
+
+  /* FIXME: Remaining problems:
+   *  - if one of the window is transient, the windows in the
+   *  background could cause problems.
+   *
+   *  - how to implement correct mouse-warping?
+   *  (is it needed at all?)
+   */
+}
+
+
 void
 blank_frame (rp_frame *frame)
 {
@@ -894,7 +949,7 @@ blank_frame (rp_frame *frame)
 }
 
 void
-hide_frame_indicator ()
+hide_frame_indicator (void)
 {
   int i;
   for (i=0; i<num_screens; i++)
@@ -902,10 +957,10 @@ hide_frame_indicator ()
 }
 
 void
-show_frame_indicator ()
+show_frame_indicator (void)
 {
   hide_frame_indicator ();
-  show_frame_message (MESSAGE_FRAME_STRING);
+  show_frame_message (defaults.frame_fmt);
   alarm (defaults.frame_indicator_timeout);
 }
 
@@ -915,10 +970,31 @@ show_frame_message (char *msg)
   rp_screen *s = current_screen ();
   int width, height;
   rp_frame *frame;
+  rp_window *win;
+  rp_window_elem *elem = NULL;
+  struct sbuf *msgbuf;
 
   frame = current_frame();
+  win = current_window ();
+  if (win)
+    {
+      rp_group *g;
 
-  width = defaults.bar_x_padding * 2 + XTextWidth (defaults.font, msg, strlen (msg));
+      g = groups_find_group_by_window (win);
+      elem = group_find_window (&g->mapped_windows, win);
+    }
+
+  /* A frame doesn't always contain a window. */
+  msgbuf = sbuf_new (0);
+  if (elem)
+    format_string (msg, elem, msgbuf);
+  else
+    {
+      sbuf_concat (msgbuf, EMPTY_FRAME_MESSAGE);
+    }
+
+  width = defaults.bar_x_padding * 2 + XmbTextEscapement (defaults.font, msgbuf->data,
+							  msgbuf->len);
   height = (FONT_HEIGHT (defaults.font) + defaults.bar_y_padding * 2);
 
   /* We don't want another frame indicator to be displayed on another
@@ -935,10 +1011,12 @@ show_frame_message (char *msg)
   XClearWindow (dpy, s->frame_window);
   XSync (dpy, False);
 
-  XDrawString (dpy, s->frame_window, s->normal_gc,
-               defaults.bar_x_padding,
-               defaults.bar_y_padding + defaults.font->max_bounds.ascent,
-               msg, strlen (msg));
+  XmbDrawString (dpy, s->frame_window, defaults.font, s->normal_gc,
+		 defaults.bar_x_padding,
+		 defaults.bar_y_padding + rp_font_ascent,
+		 msgbuf->data, msgbuf->len);
+
+  sbuf_free (msgbuf);
 }
 
 rp_frame *
