@@ -28,7 +28,7 @@ void cleanup_frame (rp_frame *frame);
 void maximize_all_windows_in_frame (rp_frame *frame);
 void h_split_frame (rp_frame *frame, int pixels);
 void v_split_frame (rp_frame *frame, int pixels);
-void remove_all_splits ();
+void remove_all_splits (void);
 void resize_shrink_to_window (rp_frame *frame);
 void resize_frame_horizontally (rp_frame *frame, int diff);
 void resize_frame_vertically (rp_frame *frame, int diff);
@@ -37,13 +37,14 @@ rp_window *find_window_for_frame (rp_frame *frame);
 rp_frame *find_windows_frame (rp_window *win);
 rp_frame *find_frame_next (rp_frame *frame);
 rp_frame *find_frame_prev (rp_frame *frame);
-rp_window *current_window ();
-void init_frame_lists ();
+rp_window *current_window (void);
+void init_frame_lists (void);
 void init_frame_list (rp_screen *screen);
 void set_active_frame (rp_frame *frame);
+void exchange_with_frame (rp_screen *s, rp_frame *cur, rp_frame *frame);
 void blank_frame (rp_frame *frame);
-void show_frame_indicator ();
-void hide_frame_indicator ();
+void show_frame_indicator (void);
+void hide_frame_indicator (void);
 
 void show_frame_message (char *msg);
 
@@ -51,9 +52,9 @@ rp_frame *find_frame_right (rp_frame *frame);
 rp_frame *find_frame_left (rp_frame *frame);
 rp_frame *find_frame_down (rp_frame *frame);
 rp_frame *find_frame_up (rp_frame *frame);
-rp_frame *find_last_frame ();
+rp_frame *find_last_frame (void);
 rp_frame * find_frame_number (int num);
 
-rp_frame *current_frame ();
+rp_frame *current_frame (void);
 
 #endif
